/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.base;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Category;

import org.experlog.gencode.DynHtml;
import org.experlog.gencode.ESDynHtml;
import org.experlog.util.ESCategory;
import org.experlog.util.ESHttpUtils;


/**
 * An openeas servlet.
 */
public class ESServlet extends HttpServlet {

	private static final long serialVersionUID = -3776044759938057487L;
	private boolean actionStatus_ = true;
    private boolean autoconnect_ = true;

    public Category logger;

    public void init(ServletConfig cfg) throws ServletException {
        try {
            super.init(cfg);
            logger = ESCategory.getCategory("openeas.servlet");
            logger.debug("configure");
            ESStore.getInstance(cfg);
        } catch (Exception e) {
            logger = ESCategory.getCategory("openeas.servlet");
            logger.error("ESServlet:init Exception " + e);
            throw new ServletException(e.getMessage());
        }
    }

    public void setAutoconnect(boolean ac) {
        autoconnect_ = ac;
    }

    public void setNoStore(HttpServletResponse res) {
        res.setHeader("Pragma", "no-cache"); // HTTP 1.0
        res.setHeader("Cache-Control", "no-store"); // HTTP 1.1
        res.setDateHeader("Expires", 0); //prevents caching at the proxy server
    }

    public void setNoContent(HttpServletResponse res) {
        res.setStatus(HttpServletResponse.SC_NO_CONTENT);
    }

    public void setNotModified(HttpServletResponse res) {
        res.setStatus(HttpServletResponse.SC_NOT_MODIFIED);
    }

    public void doGet(HttpServletRequest req, HttpServletResponse res)
    throws ServletException, IOException {
      OpenClient session = null;

      try {

        res.setStatus(HttpServletResponse.SC_OK);
        // Default is OK (HTTP 200)

        if (autoconnect_) {
          String sessionid = ESHttpUtils.getSessionId(req);
          session = ESStore.getInstance().getClient(sessionid, req);
        }

        String contentType = req.getParameter("ContentType");
        if (contentType != null) {
          res.setContentType(ESStore.getInstance().getShopConfiguration().getContentType(contentType));
        } else
          res.setContentType("text/html");

        String httpStatus = req.getParameter("http.Status");
        if(httpStatus != null) {
          try {
            res.setStatus(Integer.parseInt(httpStatus));
          } catch(Exception e) {
            res.setStatus(HttpServletResponse.SC_OK);
          }
        }

        ESServletRequest ereq = new ESServletRequest(req, res, session);

        logger = ESCategory.getCategory("openeas.servlet");
        logger.debug("ESServlet:doGet session=<" + session + ">");

        if (session != null) {

          Hashtable param = new Hashtable();
          for (Enumeration enu = req.getParameterNames();
          enu.hasMoreElements();
          ) {
            String cl = (String) enu.nextElement();
            param.put(cl, req.getParameter(cl));
          }
          session.setParam(param);

          if (session.getAppConfiguration().useCookies()) {
            javax.servlet.http.HttpSession hs = req.getSession(); 
            hs.setMaxInactiveInterval(
                session.getAppConfiguration().getCookieTimeout());
            ((ESCookie) (session.getCookie())).fromHttpSession(hs, session.getAppConfiguration(), req, res);
          }

          session.resetTimer();

          if (session.multithread())
            sget(session, ereq, res, true);
          else
            syncsget(session, ereq, res, true);

          if (session.getAppConfiguration().useCookies())
            ((ESCookie) (session.getCookie())).toHttpSession(
                req.getSession());

        } else {
          // Session is null, assume multithread is supported
          sget(session, ereq, res, true);
        }

      } catch (Exception e) {
        printException("GET", e, res);
      }
    } // method doGet

    public void getRequest(
        OpenClient session,
        ESServletRequest req,
        HttpServletResponse res)
        throws ServletException, IOException {
        logger = ESCategory.getCategory("openeas.servlet");
        logger.warn(
            "ESServlet:getRequest WARNING: ESServlet.getRequest() not overridden");

    } // method getRequest

    public void postRequest(
        OpenClient session,
        ESServletRequest req,
        HttpServletResponse res)
        throws ServletException, IOException {
        logger = ESCategory.getCategory("openeas.servlet");
        logger.warn("WARNING: ESServlet.postRequest() not overridden");

    } // method postRequest

    public void sget(
        OpenClient session,
        ESServletRequest ereq,
        HttpServletResponse res,
        boolean get)
        throws Exception {
        actionStatus_ = true;
        String action = ereq.getParameter("Action");
        if (action != null) {
            ESAction act = (ESAction) Class.forName(action).newInstance();
            if (act.serverSide()) {
                throw new ServletException(
                    "ACCESS DENIED: "
                        + action
                        + " is a server-side only action");
            }
            logger = ESCategory.getCategory("openeas.servlet");
            logger.debug("ESServlet:sget " + act.toString());

            actionStatus_ = act.processRequest(session, ereq, null, null);
        }

        // All params that end with "_" will be added to the "cookie" info
        if (session != null)
             ((ESCookie) (session.getCookie())).setCookie(ereq);
        if (get)
            getRequest(session, ereq, res);
        else
            postRequest(session, ereq, res);

    }

    public synchronized void syncsget(
        OpenClient session,
        ESServletRequest ereq,
        HttpServletResponse res,
        boolean get)
        throws Exception {
        sget(session, ereq, res, get);
    }

    public void doPost(HttpServletRequest req, HttpServletResponse res)
        throws ServletException, IOException {
        OpenClient session = null;

        try {

            res.setStatus(HttpServletResponse.SC_OK);
            // Default is OK (HTTP 200)

            if (autoconnect_) {
                String sessionid = ESHttpUtils.getSessionId(req);
                session = ESStore.getInstance().getClient(sessionid, req);
            }

			String contentType = req.getParameter("ContentType");
			if (contentType != null) {
				res.setContentType(ESStore.getInstance().getShopConfiguration().getContentType(contentType));
			} else
				res.setContentType("text/html");
			
	        String httpStatus = req.getParameter("http.Status");
	        if(httpStatus != null) {
	          try {
	            res.setStatus(Integer.parseInt(httpStatus));
	          } catch(Exception e) {
	            res.setStatus(HttpServletResponse.SC_OK);
	          }
	        }

            ESServletRequest ereq = new ESServletRequest(req, res, session);

            logger = ESCategory.getCategory("openeas.servlet");
            logger.debug("ESServlet:doGet session=<" + session + ">");

            if (session != null) {

                Hashtable param = new Hashtable();
                for (Enumeration enu = req.getParameterNames();
                    enu.hasMoreElements();
                    ) {
                    String cl = (String) enu.nextElement();
                    param.put(cl, req.getParameter(cl));
                }
                session.setParam(param);

                if (session.getAppConfiguration().useCookies()) {
                  javax.servlet.http.HttpSession hs = req.getSession(); 
                  hs.setMaxInactiveInterval(
                    session.getAppConfiguration().getCookieTimeout());
                  ((ESCookie) (session.getCookie())).fromHttpSession(hs, session.getAppConfiguration(), req, res);
                }

                session.resetTimer();

                if (session.multithread())
                    sget(session, ereq, res, true);
                else
                    syncsget(session, ereq, res, true);

                if (session.getAppConfiguration().useCookies())
                    ((ESCookie) (session.getCookie())).toHttpSession(
                        req.getSession());

            } else {
                // Session is null, assume multithread is supported
                sget(session, ereq, res, true);
            }
        } catch (Exception e) {
            printException("POST", e, res);
        }
    } // method doPost

    public boolean actionFailed() {
        return !actionStatus_;
    }
    public void printException(
        String type,
        Exception e,
        HttpServletResponse res)
        throws IOException {
        System.err.println(
            "["
                + new java.util.Date(System.currentTimeMillis())
                + "] ERROR IN ESSservlet ("
                + type
                + "), sending msg to the user's Web Browser");
        e.printStackTrace(System.err);

        if (ESStore.getInstance().getShopConfiguration().getOfftmpl() != null) {
            PrintWriter out = res.getWriter();
            out.println("<HTML><BODY>");
            DynHtml expander =
                new ESDynHtml(
                    null,
                    new File(ESStore.getInstance().getShopConfiguration().getOfftmpl()));
            try { expander.expand(out); } catch(Exception ignore) { }
            //	    e.printStackTrace(out);
            //	    e.printStackTrace(System.err);
            out.println("</BODY></HTML>");

        }
    }
    /*
    	public Administration getAdmin() {
    		if (ESStoreManager.getAdminStatus())
    			return ESStoreManager.getAdmin();
    		else
    			return null;
    	}
    	*/
}
