/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.openeas.api;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.servlet.http.HttpServletRequest;

import org.experlog.base.ESCookie;

import org.experlog.base.OpenClient;
import org.experlog.util.Client;

/**
 * A web client session
 */
public class Session {

  OpenClient session_;

  protected Session(Client s) {
    session_ = (OpenClient)s;
  }

  protected OpenClient getSession() { return session_; }

  public DbObject newDbObject(String table, ResultSet rs) throws SQLException {
    return new DbObject(new org.experlog.db.ESDbObject(session_, table, rs));
  }

  /**
   * Get the application configuration
   */
  public AppConfiguration getAppConfiguration() {
    return new AppConfiguration(session_.getAppConfiguration());
  }

  /**
   * Get the client info
   * @return The current client info.
   */
  public Cookie getCookie() {
    return new Cookie((ESCookie)session_.getCookie());
  }

  /**
   * Get the session Id (to recognize who is connected)
   * @return The session id (a String that identifies the client).
   */
  public String getClientId() { return session_.getClientId(); }

  /**
   * Find an existing session given a session id.
   * @return The requested session, or null if it does not exist.
   */
  public static Session findClient(String id) {
    OpenClient c = org.experlog.base.ESStore.getInstance().findClient(id);
    if (c == null) return null;
    return new Session(c);
  }

  /**
   * Find or create a session given a session id.
   * If the session already exists, it is returned, otherwise created.
   * @return The requested session
   */
  public static Session getClient(String id, HttpServletRequest req)
  throws Exception {
    OpenClient c = org.experlog.base.ESStore.getInstance().getClient(id, req);
    if (c == null) return null;
    return new Session(c);
  }

  /**
   * Obtain a database connection from the pool
   * @param datasourcename
   * @return a database connection
   * @throws Exception
   */
  public Connection getConnection(String datasourcename) throws Exception {
    return session_.getConnection(datasourcename);
  }

  /**
   * Send a database connection back to the pool (logical "close")
   * @param c the database connection to "close"
   * @param dsn the datasource name
   * @throws SQLException
   */
  public void closeConnection(Connection c, String dsn) throws SQLException {
    session_.closeConnection(c, dsn);
  }

  public void logRequest(String dsn, String request) {
    session_.logRequest(dsn, request);
  }

  /**
   * Issue a database update on the default datasource
   * @param sql the SQL (update or insert) request
   */
  public void updateDB(String sql) throws SQLException, Exception {
    updateDB(null,sql);
  }

  /**
   * Issue a database update on a specified datasource
   * @param datasourcename the data source name
   * @param sql the SQL (update or insert) request
   */
  public void updateDB(String datasourcename, String sql)
  throws SQLException, Exception {
    session_.updateDB(datasourcename, sql);
  }

  public void debug(String msg) {
    System.err.println(msg);
  }
  public void trace(String msg) {
    System.err.println(msg);
  }


  public String sqlEncode(String s) {
    return sqlEncode(null, s);
  }

  public String sqlEncode(String datasourcename, String s) {
    return session_.sqlEncode(datasourcename, s);
  }

  public void resetTimer() {
    session_.resetTimer();
  }

};

