/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;

import org.experlog.openeas.api.DataObject;

public class ESDbMapper implements DataObject {

	Hashtable tab_ = null;

	public ESDbMapper(String fname) throws IOException {

		if (fname == null) return;

		Properties props = null;
		FileInputStream in = null;
		try {
			in = new FileInputStream(new File(fname));
			props = new Properties();
			props.load(in);
			in.close();
		} catch (IOException ioe) {
			if (in != null)
				try {
					in.close();
				} catch (IOException ee) {
					System.err.println("ESDbMapper Exception " + ee);
				}
			throw ioe;
		}

		tab_ = new Hashtable(props.size());
		Enumeration en = props.keys();
		while (en.hasMoreElements()) {
			String key = en.nextElement().toString();
			tab_.put(key.toUpperCase(), props.get(key));
		}
	}

	public String getTable(String table) {
		try {
			return get(table);
		} catch (SQLException e) {
			return table;
		}
	}

	public String getField(String table, String field) {
		try {
			return get(table + "." + field);
		} catch (SQLException e) {
			return field;
		}
	}

	/**
	 * Method from ESDataTuple interface
	 */
	public String get(String name) throws SQLException {
		if (tab_ == null) return getName(name);
		String newname = (String) tab_.get(name.toUpperCase());
		if (newname == null) return getName(name);
		return newname;
	}

	private String getName(String name) {
		int pos = name.indexOf('.');
		if (pos >= 0)
			return name.substring(pos + 1);
		else
			return name;
	}

	protected void finalize() {
		if (tab_ != null) tab_.clear();
		tab_ = null;
	}

}

